@@ -1,5 +1,5 @@
 #  ##### BEGIN GPL LICENSE BLOCK #####
-# 
+#
 #  This program is free software: you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
 #  the Free Software Foundation, either version 3 of the License, or
@@ -17,6 +17,8 @@
 
 import pygame
 
+pygame.init()
+
 class ButtonText:
     """
     class ButtonText: General purpose text-based button.
@@ -53,22 +55,22 @@ class ButtonText:
         self.clickButton = clickButton
         self.clicked = False
 
-    def Draw(self, window):
+    def Draw(self, window, events):
         """
         Draws button on a window.
         :param window: Pygame window object to draw on.
+        :param events: Events obtained from pygame.events.get()
         """
         pygame.draw.rect(window, self.currBgCol, self.loc + self.size)
         window.blit(self.text, self.textLoc)
         if self.border:
             pygame.draw.rect(window, self.borderCol, self.loc+self.size, self.borderWidth)
 
-    def Update(self):
-        """
-        Updates button info like clicked and color.
-        """
         mouse = pygame.mouse.get_pos()
-        click = pygame.mouse.get_pressed()[self.clickButton]
+        click = False
+        for event in events:
+            if event.type == pygame.MOUSEBUTTONDOWN:
+                click = True
 
         loc = self.loc
         size = self.size
@@ -85,40 +87,44 @@ class ButtonText:
 
 
 class TextInput:
-    def __init__(
-            self,
-            loc=(0, 0),
-            size=(100, 50),
-            bgCol=(255, 255, 255),
-            borderWidth=5,
-            borderCol=(0, 0, 0),
-            initialText="",
-            fontName="comicsans",
-            fontSize=35,
-            textCol=(0, 0, 0),
-            cursorCol=(0, 0, 1),
-            repeatInitial=400,
-            repeatInterval=35,
-            maxLen=-1,
-            password=False,
-            editing=False,
-            ):
+    def __init__(self, loc, size, bgCol, borderWidth=5, borderCol=(0, 0, 0), initialText="", label="", font=pygame.font.SysFont("comicsans", 35), \
+        textCol=(0, 0, 0), cursorCol=(0, 0, 1), repeatInitial=400, repeatInterval=35, maxLen=-1, password=False, editing=False):
+        """
+        Input text class for Pygame.
+        :param loc: Pixel location (x, y).
+        :param size: Pixel size (x, y).
+        :param bgCol: Background color (RGB).
+        :param borderWidth: Width of border (pixels).
+        :param borderCol: Color of border (RGB).
+        :param initialText: Text to start with.
+        :param label: Label (name) of input.
+        :param font: pygame.font object.
+        :param textCol: Color of text.
+        :param cursorCol: Color of cursor.
+        :param repeatInitial: Initial delay (ms) for repeating (pressed down keys).
+        :param repeatInterval: Delay between repeats (pressed down keys).
+        :param maxLen: Maximum length of string (-1 = inf)
+        :param password: Boolean specifying whether to display as asterisks (*).
+        :param editing: Default to enter edit mode.
+        """
+
+        self.passwordField = password
 
         self.loc, self.size = loc, size
 
         self.editing = editing
 
         self.textCol = textCol
-        self.fontSize = fontSize
         self.password = password
         self.text = initialText
+        self.label = label
         self.maxLen = maxLen
 
         self.rect = pygame.Rect(*loc, *size)
         self.bgCol = bgCol
         self.borderCol, self.borderWidth = borderCol, borderWidth
 
-        self.font = pygame.font.SysFont(fontName, fontSize)
+        self.font = font
 
         self.surface = pygame.Surface((1, 1))
         self.surface.set_alpha(0)
@@ -127,7 +133,7 @@ class TextInput:
         self.keyrepeatInitial = repeatInitial
         self.keyrepeatInterval = repeatInterval
 
-        self.cursorSurf = pygame.Surface((int(fontSize / 20 + 1), fontSize))
+        self.cursorSurf = pygame.Surface((int(font.get_height() / 20 + 1), font.get_height()))
         self.cursorSurf.fill(cursorCol)
         self.cursorPos = len(initialText)
         self.cursorVisible = True
@@ -136,7 +142,7 @@ class TextInput:
 
         self.clock = pygame.time.Clock()
 
-    def Draw(self, window):
+    def Draw(self, window, events):
         pygame.draw.rect(window, self.bgCol, self.rect)
         if self.borderWidth:
             pygame.draw.rect(window, self.borderCol, self.rect, self.borderWidth)
@@ -144,7 +150,6 @@ class TextInput:
         textPos = (int(self.loc[0] + self.size[0]//2 - self.surface.get_width()/2), int(self.loc[1] + self.size[1]//2 - self.surface.get_height()/2))
         window.blit(self.surface, textPos)
 
-    def Update(self, events):
         for event in events:
             if event.type == pygame.MOUSEBUTTONDOWN:
                 if self.rect.collidepoint(event.pos):
@@ -152,6 +157,14 @@ class TextInput:
                 else:
                     self.editing = False
 
+            if not self.text:
+                self.password = False
+                self.text = self.label
+
+            if self.editing and self.text == self.label:
+                self.ClearText()
+                self.password = True if self.passwordField else False
+
             if event.type == pygame.KEYDOWN:
                 self.cursorVisible = True
 
@@ -232,6 +245,7 @@ class TextInput:
         self.clock.tick()
         return False
 
+
     def GetCursorPos(self):
         return self.cursorPos
 
@@ -246,4 +260,41 @@ class TextInput:
         self.cursorPos = 0
 
     def __repr__(self):
-        return self.text
\ No newline at end of file
+        return self.text
+
+
+class Slider:
+    def __init__(self, rectLoc, rectSize, rectCol=(128, 128, 128), circleCol=(255, 255, 255), valRange=(1, 100), initialVal=20, horiz=True):
+        """
+        Creates a slider with various tweakable parameters.
+
+        :param rectLoc: The (x, y) location of the rectangle underneath the circle.
+        :type rectLoc: tuple
+        :param rectSize: The (width, height) size of the rectangle underneath the circle.
+        :type rectSize: tuple
+        :param rectCol: The (R, G, B) color of the rectangle underneath the circle.
+        :type rectCol: tuple
+        :param circleCol: The (R, G, B) color of the circle above the rectangle.
+        :type circleCol: tuple
+        :param valRange: A tuple indicating the range of values that the slider can scroll through (min, max).
+        :type valRange: tuple
+        :param initialVal: The starting value of the slider.
+        :type initialVal: int
+        :param horiz: A boolean indicating whether or not the slider is horizontal(True) or vertical(False)
+        :type horiz: boolean
+        """
+
+        self.loc, self.size = rectLoc, rectSize
+        self.rectCol, self.circleCol = rectCol, circleCol
+        self.valRange = valRange
+        self.value = initialVal
+        self.horiz = horiz
+
+    def GetValue(self):
+        """
+        Returns the current value of the slider.
+
+        :return: Current value of the slider
+        :rtype: int
+        """
+        return self.value
